@@ -1,15 +1,5 @@
-# $Id: Item.pm 62129 2008-10-01 22:52:20Z wsnyder $
-######################################################################
-#
-# Copyright 2001-2008 by Wilson Snyder.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
+# SystemC - SystemC Perl Interface
+# See copyright, etc in below POD section.
 ######################################################################
 
 package SystemC::Coverage::Item;
@@ -22,7 +12,7 @@ use vars qw($VERSION $Debug $AUTOLOAD);
 ######################################################################
 #### Configuration Section
 
-$VERSION = '1.300';
+$VERSION = '1.310';
 
 ######################################################################
 ######################################################################
@@ -80,7 +70,7 @@ sub key {
 sub hash {
     # Return hash of all keys and values
     my %hash;
-    while ($_[0]->[0] =~ /\001([^\002]+)\002([^\001]+)/g) {
+    while ($_[0]->[0] =~ /\001([^\002]+)\002([^\001]*)/g) {
 	my $key=$SystemC::Coverage::ItemKey::DecompressKey{$1}||$1;
 	$hash{$key}=$2;
     }
@@ -98,7 +88,7 @@ sub write_string {
     my $self = shift;
     my $str = "inc(";
     my $comma = "";
-    while ($self->[0] =~ /\001([^\002]+)\002([^\001]+)/g) {
+    while ($self->[0] =~ /\001([^\002]+)\002([^\001]*)/g) {
 	my $key = $1;
 	my $val = $2;
 	$key = "'".$key."'" if length($key)!=1;
@@ -124,7 +114,7 @@ sub AUTOLOAD {
 	else { $def = "'$def'"; }
 	my $f = ("package SystemC::Coverage::Item;"
 		 ."sub $func {"
-		 ."  if (\$_[0]->[0] =~ /\\001${key}\\002([^\\001]+)/) {"
+		 ."  if (\$_[0]->[0] =~ /\\001${key}\\002([^\\001]*)/) {"
 		 ."    return \$1;"
 		 ."  } else {"
 		 ."    return ".$def.";"
@@ -254,7 +244,7 @@ SystemPerl is part of the L<http://www.veripool.org/> free SystemC software
 tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/systemperl>.
 
-Copyright 2001-2008 by Wilson Snyder.  This package is free software; you
+Copyright 2001-2009 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License or the Perl Artistic License.
 
