@@ -1,15 +1,5 @@
-# $Id: ItemKey.pm 62129 2008-10-01 22:52:20Z wsnyder $
-######################################################################
-#
-# Copyright 2001-2008 by Wilson Snyder.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
+# SystemC - SystemC Perl Interface
+# See copyright, etc in below POD section.
 ######################################################################
 
 package SystemC::Coverage::ItemKey;
@@ -21,7 +11,7 @@ use vars qw($VERSION %CompressKey %DecompressKey %GroupKey);
 ######################################################################
 #### Configuration Section
 
-$VERSION = '1.300';
+$VERSION = '1.310';
 
 our %_Keys =
     (
@@ -34,6 +24,7 @@ our %_Keys =
      "filename"	 => { compressed=>"f",	group=>1, default=>undef, },
      "groupdesc" => { compressed=>"d",	group=>1, default=>"", },
      "groupname" => { compressed=>"g",	group=>1, default=>"", },
+     "groupcmt"	 => { compressed=>"O",	group=>1, default=>"", },
      "hier"	 => { compressed=>"h",	group=>1, default=>"", },
      "per_instance"=>{compressed=>"P",	group=>1, default=>0,	},
      "row0_name" => { compressed=>"R0", group=>1, default=>undef, },
@@ -177,7 +168,7 @@ SystemPerl is part of the L<http://www.veripool.org/> free SystemC software
 tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/systemperl>.
 
-Copyright 2001-2008 by Wilson Snyder.  This package is free software; you
+Copyright 2001-2009 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License or the Perl Artistic License.
 
