@@ -1,17 +1,5 @@
 # SystemC - SystemC Perl Interface
-# $Id: CoverPoint.pm 62129 2008-10-01 22:52:20Z wsnyder $
-# Author: Bobby Woods-Corwin <me@alum.mit.edu>
-######################################################################
-#
-# Copyright 2001-2008 by Bobby Woods-Corwin.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
+# See copyright, etc in below POD section.
 ######################################################################
 
 package SystemC::Netlist::CoverPoint;
@@ -22,13 +10,20 @@ use Verilog::Netlist;
 use Verilog::Netlist::Subclass;
 @ISA = qw(SystemC::Netlist::CoverPoint::Struct
 	  Verilog::Netlist::Subclass);
-$VERSION = '1.300';
+$VERSION = '1.310';
 use strict;
 
+# allow 64-bit values without bonking
+no warnings 'portable';
+
 # The largest value for which we will use the faster lookup table
 # to compute bin number (as opposed to if statements)
 use constant MAX_BIN_LOOKUP_SIZE => 256;
 
+# longest allowed user-defined string
+#use constant MAX_USER_STRING_LEN => 256;
+use constant MAX_USER_STRING_LEN => 5000;
+
 struct('Bin'
        =>[name          => '$', #'	# name of bin
 	  ranges   	=> '@', #'	# ranges
@@ -50,7 +45,11 @@ structs('new',
 	   maxValue     => '$', #'	# max specified value
 	   minValue     => '$', #'	# min specified value
 	   enum         => '$', #'	# if an enum, what's the enum name?
+	   ignoreFunc   => '$', #'	# if present, the function name to compute ignores
+	   illegalFunc  => '$', #'	# if present, the function name to compute illegals
 	   isCross      => '$', #'	# is this point a cross
+	   crossMember  => '$', #'	# is this point a member of another cross
+	   radix        => '$', #'	# for standard bins, with what radix to number them
 	   rows         => '@', #'	# (cross) list of rows
 	   cols   	=> '@', #'	# (cross) list of columns
 	   tables   	=> '@', #'	# (cross) list of tables
@@ -95,9 +94,11 @@ sub current_coverpoint {
 	     num_bins => 1,
 	     name     => "",
 	     description => "",
-	     defaultName => "default",
+	     defaultName => "",
 	     maxValue => 0,
 	     minValue => 0,
+	     crossMember => 0,
+	     radix => 10,
 	     );
 	$self->attributes("_openCoverpoint",$coverpointref);
     }
@@ -128,7 +129,9 @@ sub coverpoint_sample_text {
     my $pointname = $self->name;
     my $out;
 
-    if ($self->isCross) {
+    if ($self->crossMember) {
+	$out .= "/* point name = $pointname is a crossMember - no separate sample needed */\n";
+    } elsif ($self->isCross) {
 	$out .= "/* cross name = $pointname */\n";
 	$out .= "{ ++_sp_cg_".$groupname."_".$pointname;
 
@@ -173,6 +176,7 @@ sub cross_build {
 	my $currentCovergroup = $self->module->current_covergroup();
 	foreach my $point (@{$currentCovergroup->coverpoints}) {
 	    if ($point->name eq $item) {
+		$point->crossMember(1);
 		my $dimension = $self->attributes("dimension");
 		if ($dimension eq "rows") {
 		    push @{$self->rows}, $point;
@@ -200,13 +204,8 @@ sub coverpoint_build {
     if ($type eq "binval") {
 	my $val_str = shift;
 	print "Netlist::File: coverpoint parsed binval: $val_str\n" if $SystemC::Netlist::Debug;
-	my $val = $self->validate_value($val_str);
-
 	my $bin = $self->current_bin();
-	push @{$bin->values}, $val;
-
-	if ($val < $self->minValue) { $self->minValue($val);}
-	if ($val > $self->maxValue) { $self->maxValue($val);}
+	push @{$bin->values}, $val_str;
 
 	if ($self->attributes("in_multi_bin")) {
 
@@ -229,16 +228,8 @@ sub coverpoint_build {
 	my $lo_str = shift;
 	my $hi_str = shift;
 	print "Netlist::File: coverpoint parsed binrange: $lo_str:$hi_str\n" if $SystemC::Netlist::Debug;
-	my $lo = $self->validate_value($lo_str);
-	my $hi = $self->validate_value($hi_str);
-
 	my $bin = $self->current_bin();
-	push @{$bin->ranges}, "$hi:$lo";
-
-	if ($lo < $self->minValue) { $self->minValue($lo);}
-	if ($hi < $self->minValue) { $self->minValue($hi);}
-	if ($lo > $self->maxValue) { $self->maxValue($lo);}
-	if ($hi > $self->maxValue) { $self->maxValue($hi);}
+	push @{$bin->ranges}, "$hi_str,$lo_str";
 
 	if ($self->attributes("in_multi_bin")) {
 	    $bin->name($self->attributes("multi_bin_basename")
@@ -269,9 +260,21 @@ sub coverpoint_build {
 	$bin->name($binname);
 	$self->attributes("in_illegal",0);
 	$self->attributes("in_ignore",1);
+    } elsif ($type eq "ignore_func") {
+	my $func = shift;
+	$self->ignoreFunc($func);
+    } elsif ($type eq "illegal_func") {
+	my $func = shift;
+	$self->illegalFunc($func);
     } elsif ($type eq "normal") {
 	my $binname = shift;
 	print "Netlist::File: coverpoint parsed normal bin, name = $binname\n" if $SystemC::Netlist::Debug;
+
+	if (length($binname) > MAX_USER_STRING_LEN) {
+	    my $max = MAX_USER_STRING_LEN;
+	    $self->error ("SP_COVERGROUP \"$binname\" string too long (max $max chars)\n");
+	}
+
 	my $bin = $self->current_bin();
 	$bin->name($binname);
 	$self->attributes("in_illegal",0);
@@ -291,9 +294,40 @@ sub coverpoint_build {
 	$self->attributes("in_multi_bin",1);
 	$self->attributes("multi_bin_count",0);
 	$self->attributes("multi_bin_basename",$bin->name);
+    } elsif ($type eq "multi_begin_num") {
+	my $num_ranges = shift;
+	print "Netlist::File: coverpoint parsed multi_begin_num\n" if $SystemC::Netlist::Debug;
+	my $bin = $self->current_bin();
+	$self->attributes("in_multi_bin",1);
+	$self->attributes("multi_bin_num_ranges",$num_ranges); # we use this in multi_bin_end
+	$self->attributes("multi_bin_count",0);
+	$self->attributes("multi_bin_basename",$bin->name);
     } elsif ($type eq "multi_end") {
 	print "Netlist::File: coverpoint parsed multi_end\n" if $SystemC::Netlist::Debug;
 	$self->attributes("in_multi_bin",0);
+    } elsif ($type eq "multi_auto_end") {
+	print "Netlist::File: coverpoint parsed multi_auto_end\n" if $SystemC::Netlist::Debug;
+	$self->attributes("in_multi_bin",0);
+
+	# there's a single bin with a range; we want to
+	# convert it into a bunch of individual bins of size 1
+	my $bin = pop @{$self->bins};
+
+	my $range = pop @{$bin->ranges};
+	$range =~ /(\S+),(\S+)/;
+
+	my $hi_str = $1;
+	my $lo_str = $2;
+	my $lo = $self->validate_value($lo_str,$fileref);
+	my $hi = $self->validate_value($hi_str,$fileref);
+
+	if ($self->attributes("multi_bin_num_ranges")) {
+	    $self->make_standard_bins($self->attributes("multi_bin_num_ranges")
+				      ,$lo,$hi,$self->attributes("multi_bin_basename"));
+	} else {
+	    $self->make_standard_bins(($hi - $lo + 1),$lo,$hi,$self->attributes("multi_bin_basename"));
+	}
+	$self->attributes("multi_bin_num_ranges",0);
     } elsif ($type eq "standard") {
 	print "Netlist::File: coverpoint parsed standard\n" if $SystemC::Netlist::Debug;
 	# only the default bin
@@ -303,18 +337,18 @@ sub coverpoint_build {
 	my $lo_str = shift;
 	my $hi_str = shift;
 	print "Netlist::File: coverpoint parsed standard_bins_range, size = $binsize_str, lo = $lo_str, hi = $hi_str\n" if $SystemC::Netlist::Debug;
-	my $binsize = $self->validate_value($binsize_str);
-	my $lo = $self->validate_value($lo_str);
-	my $hi = $self->validate_value($hi_str);
+	my $binsize = $self->validate_value($binsize_str,$fileref);
+	my $lo = $self->validate_value($lo_str,$fileref);
+	my $hi = $self->validate_value($hi_str,$fileref);
 
-	$self->make_standard_bins($binsize,$lo,$hi);
+	$self->make_standard_bins($binsize,$lo,$hi,$self->name);
     } elsif ($type eq "standard_bins") {
 	my $binsize_str = shift;
 	print "Netlist::File: coverpoint parsed standard_bins, size = $binsize_str\n" if $SystemC::Netlist::Debug;
-	my $binsize = $self->validate_value($binsize_str);
+	my $binsize = $self->validate_value($binsize_str,$fileref);
 	# FIXME default 1024 is a hack
 	# we should look up the size from the sp_ui etc.
-	$self->make_standard_bins($binsize,0,1023);
+	$self->make_standard_bins($binsize,0,1023,$self->name);
     } elsif ($type eq "bins") {
 	print "Netlist::File: coverpoint parsed explicit bins\n" if $SystemC::Netlist::Debug;
 	$self->num_bins(scalar(@{$self->bins})+1); # +1 for default
@@ -326,11 +360,39 @@ sub coverpoint_build {
     } elsif ($type eq "page") {
 	my $page = shift;
 	print "Netlist::File: coverpoint parsed page = $page\n" if $SystemC::Netlist::Debug;
+	if (length($page) > MAX_USER_STRING_LEN) {
+	    my $max = MAX_USER_STRING_LEN;
+	    $self->error ("SP_COVERGROUP \"$page\" string too long (max $max chars)\n");
+	}
 	$self->page($page);
     } elsif ($type eq "description") {
 	my $desc = shift;
 	print "Netlist::File: coverpoint parsed description = $desc\n" if $SystemC::Netlist::Debug;
+	if (length($desc) > MAX_USER_STRING_LEN) {
+	    my $max = MAX_USER_STRING_LEN;
+	    $self->error ("SP_COVERGROUP \"$desc\" string too long (max $max chars)\n");
+	}
 	$self->description($desc);
+    } elsif ($type eq "option") {
+	my $var = shift;
+	my $val = shift;
+
+	if (length($var) > MAX_USER_STRING_LEN) {
+	    my $max = MAX_USER_STRING_LEN;
+	    $self->error ("SP_COVERGROUP \"$var\" string too long (max $max chars)\n");
+	}
+
+	if ($var eq "radix") {
+	    if (($val == 16) ||
+		($val == 10) ||
+		($val == 2)) {
+		$self->radix($val);
+	    } else {
+		$self->error("Unrecognized radix option \"$val\"; I know about 2/10/16\n");
+	    }
+	} else {
+	    $self->error("Unrecognized coverpoint option \"$var = $val\"\n");
+	}
     } else {
 	$self->error("Netlist::File: coverpoint parsed an unexpected type: $type\n");
     }
@@ -339,8 +401,7 @@ sub coverpoint_build {
 sub validate_value {
     my $self = shift;
     my $str = shift;
-
-    ## FIXME we'd like to recognize enums here too
+    my $fileref = shift;
 
     if ($str =~ /^0x[0-9a-fA-F]+$/) { # hex number
 	#print "recognized hex $str as ". (hex $str)."\n";
@@ -348,6 +409,23 @@ sub validate_value {
     } elsif ($str =~ /^\d+$/) { # decimal number
 	#print "recognized dec $str\n";
 	return $str;
+    } elsif ($str =~ /^(\w+)::(\w+)$/) { # enum
+	my $enumclass = $1;
+	my $enumname = $2;
+
+	# do we recognize the enum name?
+	my $netlist = $fileref->netlist();
+	my $vals = $netlist->{_enums}{$enumclass};
+	if (!defined $vals) {
+	    $self->error("parsed what looks like an enum but is an unrecognized enum class: ${enumclass}\n");
+	    return 0;
+	}
+	my $val = $vals->{"en"}{$enumname};
+	if (!defined $val) {
+	    $self->error("parsed a recognized enum type but an unrecognized value: ${enumclass}::${enumname}\n");
+	    return 0;
+	}
+	return $val;
     } else {
 	$self->error("parsed coverpoint bin value not a decimal or hex number: $str");
 	return 0;
@@ -360,22 +438,39 @@ sub make_standard_bins {
     my $num_bins = shift;
     my $lo_range = shift;
     my $hi_range = shift;
+    my $name = shift;
 
     my $span = $hi_range - $lo_range + 1;
 
+    if ($span < $num_bins) {
+	$self->error("more bins specified ($num_bins) than values in range ($lo_range:$hi_range)\n");
+    }
+
+    my $radix_char = "d";
+    $radix_char = "x" if ($self->radix == 16);
+    $radix_char = "d" if ($self->radix == 10);
+    $radix_char = "b" if ($self->radix == 2);
+    # add just the right number of leading zeros
+    my $s = sprintf("%".$radix_char,$num_bins-1+$lo_range);
+    my $digits = length $s;
+    my $s2 = sprintf("%".$radix_char,$num_bins-1);
+    my $name_digits = length $s2;
+
     # make bins
     for(my $i=0;$i<$num_bins;$i++) {
 	my $bin = $self->current_bin();
 	my $lo = int(($span / $num_bins) * $i) + $lo_range;
 	my $hi = (int(($span / $num_bins) * ($i+1)) - 1) + $lo_range;
-	push @{$bin->ranges}, "$hi:$lo";
 
-	if ($lo < $self->minValue) { $self->minValue($lo);}
-	if ($hi < $self->minValue) { $self->minValue($hi);}
-	if ($lo > $self->maxValue) { $self->maxValue($lo);}
-	if ($hi > $self->maxValue) { $self->maxValue($hi);}
+	push @{$bin->ranges}, sprintf("%u,%u",$hi,$lo); # force unsigned
+
+	if ($span == $num_bins) {
+	    # no names required!
+	    $bin->name(sprintf("%0".$digits.$radix_char,$i+$lo_range));
+	} else {
+	    $bin->name(sprintf("%s_%0".$name_digits.$radix_char,$name,$i));
+	}
 
-	$bin->name($self->name."_".$i);
 	push @{$self->bins}, $bin;
 	# undef it so that the next bin will be fresh
 	$self->attributes("_openBin",undef);
@@ -411,7 +506,7 @@ sub make_auto_enum_bins {
 	my $val = $vals->{$enumtype}{$valsym};
 
 	my $bin = $self->current_bin();
-	push @{$bin->values}, $val;
+	push @{$bin->values}, sprintf("%u",$val); # force unsigned
 	if ($val < $self->minValue) { $self->minValue($val);}
 	if ($val > $self->maxValue) { $self->maxValue($val);}
 
@@ -456,26 +551,150 @@ sub _write_coverpoint_decl {
 	foreach my $dimension (@dimensions) {
 	    $fileref->printf ("[%d]",$dimension->num_bins);
 	}
-
 	$fileref->printf (";\t// SP_COVERGROUP declaration\n");
 
-    } else { # not a cross
-	# declare the coverpoint
-	$fileref->printf ("%sSpZeroed<uint32_t>\t_sp_cg_%s_%s[%d];\t// SP_COVERGROUP declaration\n",
+	###########################################################################
+	# write the function returning the ignoredness
+	###########################################################################
+	$fileref->printf ("%sbool _sp_cg_%s_%s_ignored(",
 			  $prefix,
 			  $covergroupref->name,
-			  $self->name,
-			  $self->num_bins);
+			  $self->name);
+	my @args;
+	foreach my $dimension (@dimensions) {
+	    my $dimname = $dimension->name;
+	    push @args, "uint64_t $dimname";
+	}
+	my $argsWithCommas = join(', ',@args);
+	$fileref->printf ("%s) { \t// SP_COVERGROUP declaration\n",$argsWithCommas);
 
+	$fileref->printf ("%s  return (0 // if any dimension is ignored\n",$prefix);
+	foreach my $dimension (@dimensions) {
+	    $fileref->printf ("%s         || _sp_cg_%s_%s_ignored(%s)\n",
+			      $prefix,
+			      $covergroupref->name,
+			      $dimension->name,
+			      $dimension->name);
+	}
+	if ($self->ignoreFunc) {
+	    my @args2;
+	    foreach my $dimension (@dimensions) {
+		my $dimname = $dimension->name;
+		my $str = "_sp_cg_".$covergroupref->name."_".$dimension->name."_getArbitraryValue(${dimname})";
+		push @args2, $str;
+	    }
+	    my $args2WithCommas = join(', ',@args2);
+	    $fileref->printf ("%s         || %s(%s) // or if my func says it should be ignored\n",
+			      $prefix,$self->ignoreFunc,$args2WithCommas);
+	}
+	$fileref->printf ("%s         );\n",$prefix);
+	$fileref->printf ("%s}\n",$prefix);
+
+	###########################################################################
+	# write the function returning the illegality
+	###########################################################################
+	$fileref->printf ("%sbool _sp_cg_%s_%s_illegal(",
+			  $prefix,
+			  $covergroupref->name,
+			  $self->name);
+	my @args3;
+	foreach my $dimension (@dimensions) {
+	    my $dimname = $dimension->name;
+	    push @args3, "uint64_t $dimname";
+	}
+	$argsWithCommas = join(', ',@args3);
+	$fileref->printf ("%s) { \t// SP_COVERGROUP declaration\n",$argsWithCommas);
+
+	$fileref->printf ("%s  return (0 // if any dimension is illegal\n",$prefix);
+	foreach my $dimension (@dimensions) {
+	    $fileref->printf ("%s         || _sp_cg_%s_%s_illegal(%s)\n",
+			      $prefix,
+			      $covergroupref->name,
+			      $dimension->name,
+			      $dimension->name);
+	}
+	if ($self->illegalFunc) {
+	    my @args2;
+	    foreach my $dimension (@dimensions) {
+		my $dimname = $dimension->name;
+		my $str = "_sp_cg_".$covergroupref->name."_".$dimension->name."_getArbitraryValue(${dimname})";
+		push @args2, $str;
+	    }
+	    my $args2WithCommas = join(', ',@args2);
+	    $fileref->printf ("%s         || %s(%s) // or if my func says it should be illegal\n",
+			      $prefix,$self->illegalFunc,$args2WithCommas);
+	}
+	$fileref->printf ("%s          );\n",$prefix);
+	$fileref->printf ("%s}\n",$prefix);
+
+    } else { # not a cross
+	if (!$self->crossMember) {
+	    # declare the coverpoint (only if not a crossMember)
+	    $fileref->printf ("%sSpZeroed<uint32_t>\t_sp_cg_%s_%s[%d];\t// SP_COVERGROUP declaration\n",
+			      $prefix,
+			      $covergroupref->name,
+			      $self->name,
+			      $self->num_bins);
+	}
+
+	###########################################################################
+	# write the function returning an arbitrary value per bin
+	###########################################################################
+	$fileref->printf ("%suint64_t _sp_cg_%s_%s_getArbitraryValue(uint64_t bin) { \t// SP_COVERGROUP declaration\n",
+			  $prefix,
+			  $covergroupref->name,
+			  $self->name);
+
+	my $bin_num = 0;
+	foreach my $bin (@{$self->bins}) {
+	    if (scalar @{$bin->values}) {
+		my @vals = @{$bin->values};
+		my $arbitrary_val = $vals[0];
+
+		# if it's not an enum, then add ULL to allow 64-bit numbers
+		$arbitrary_val .= "ULL" unless ($arbitrary_val =~ /^(\w+)::(\w+)$/);
+
+		$fileref->printf ("%s  if (bin == %s) return %s; // an arbitrary value in bin %s\n",
+				  $prefix,$bin_num,$arbitrary_val,$bin->name);
+	    } elsif (scalar @{$bin->ranges}) {
+		my @ranges = @{$bin->ranges};
+		$ranges[0] =~ /(\S+),(\S+)/;
+		my $hi_str = $1;
+
+		# if it's not an enum, then add ULL to allow 64-bit numbers
+		$hi_str .= "ULL" unless ($hi_str =~ /^(\w+)::(\w+)$/);
+
+		$fileref->printf ("%s  if (bin == %s) return %s; // an arbitrary value in bin %s\n",
+				  $prefix,$bin_num,$hi_str,$bin->name);
+	    } else {
+		my $binname = $bin->name;
+		$self->error("CoverPoint internal error: bin $binname has no values or ranges!\n");
+	    }
+	    $bin_num++;
+	}
+	if ($self->defaultName eq "") {
+	    $fileref->printf ("%s  if (bin == %s) return 0; // the unnamed default bin - the return value doesn't matter\n",
+			      $prefix,$bin_num);
+	} else {
+	    $fileref->printf ("%s  if (bin == %s) return 0; // the default bin (%s) - the return value doesn't matter\n",
+			      $prefix,$bin_num,$self->defaultName);
+	}
+ 	$fileref->printf ("%s  SP_ERROR_LN(\"%s\",%d,\"Internal error: Illegal bin value for point %s\");\n",
+			  $prefix,$fileref->name,$covergroupref->module->lineno,$self->name);
+ 	$fileref->printf ("%s  return 0;\n", $prefix);
+	$fileref->printf ("%s}\n", $prefix);
+
+	###########################################################################
 	# write the function returning the ignoredness
-	$fileref->printf ("%sstring _sp_cg_%s_%s_ignoreStr(uint64_t bin) { \t// SP_COVERGROUP declaration\n",
+	###########################################################################
+	$fileref->printf ("%sbool _sp_cg_%s_%s_ignored(uint64_t bin) { \t// SP_COVERGROUP declaration\n",
 			  $prefix,
 			  $covergroupref->name,
 			  $self->name);
 	$fileref->printf ("%s  static int _s_bin_to_ignore[] = {",$prefix);
 	my @lookupTable = (0) x ($self->num_bins);
 
-	my $bin_num = 0;
+	$bin_num = 0;
 	foreach my $bin (@{$self->bins}) {
 	    $lookupTable[$bin_num] = $bin->isIgnore;
 	    $bin_num+=1;
@@ -485,14 +704,23 @@ sub _write_coverpoint_decl {
 	    $fileref->printf ("%d,",$lookupTable[$i]);
 	}
 	$fileref->printf ("};\n");
- 	$fileref->printf ("%s  if (bin >= %d) { SP_ERROR_LN(\"%s\",%d,\"Internal error: Illegal bin value\"); return \"ignore\"; }\n",
+	if ($self->ignoreFunc) {
+	    $fileref->printf ("%s  if (%s(_sp_cg_%s_%s_getArbitraryValue(bin))) { return true; }\n",
+			      $prefix,$self->ignoreFunc,
+			      $covergroupref->name,
+			      $self->name);
+	}
+
+ 	$fileref->printf ("%s  if (bin >= %d) { SP_ERROR_LN(\"%s\",%d,\"Internal error: Illegal bin value in %s_ignore\"); return true; }\n",
  			  $prefix,$self->num_bins,
- 			  $fileref->name,$covergroupref->module->lineno,
-			  );
-	$fileref->printf ("%s  return (_s_bin_to_ignore[bin] ? \"ignore\" : \"\");\n%s}\n",
+ 			  $fileref->name,$covergroupref->module->lineno,$self->name);
+	$fileref->printf ("%s  return (_s_bin_to_ignore[bin]);\n%s}\n",
 			  $prefix,$prefix);
+
+	###########################################################################
 	# write the function returning the illegality
-	$fileref->printf ("%sstring _sp_cg_%s_%s_illegalStr(uint64_t bin) { \t// SP_COVERGROUP declaration\n",
+	###########################################################################
+	$fileref->printf ("%sbool _sp_cg_%s_%s_illegal(uint64_t bin) { \t// SP_COVERGROUP declaration\n",
 			  $prefix,
 			  $covergroupref->name,
 			  $self->name);
@@ -510,19 +738,26 @@ sub _write_coverpoint_decl {
 	    $fileref->printf ("%d,",$lookupTable[$i]);
 	}
 	$fileref->printf ("};\n");
- 	$fileref->printf ("%s  if (bin >= %d) { SP_ERROR_LN(\"%s\",%d,\"Internal error: Illegal bin value\"); return \"illegal\"; }\n",
+	if ($self->illegalFunc) {
+	    $fileref->printf ("%s  if (%s(_sp_cg_%s_%s_getArbitraryValue(bin))) { return true; }\n",
+			      $prefix,$self->illegalFunc,
+			      $covergroupref->name,
+			      $self->name);
+	}
+ 	$fileref->printf ("%s  if (bin >= %d) { SP_ERROR_LN(\"%s\",%d,\"Internal error: Illegal bin value in %s_illegal\"); return true; }\n",
  			  $prefix,$self->num_bins,
- 			  $fileref->name,$covergroupref->module->lineno,
- 			  );
-	$fileref->printf ("%s  return (_s_bin_to_illegal[bin] ? \"illegal\" : \"\");\n%s}\n",
+ 			  $fileref->name,$covergroupref->module->lineno,$self->name);
+	$fileref->printf ("%s  return (_s_bin_to_illegal[bin]);\n%s}\n",
 			  $prefix,$prefix);
 
+	###########################################################################
 	# write the function returning the bin name
-	$fileref->printf ("%sstatic string _sp_cg_%s_%s_binName(uint64_t point) { \t// SP_COVERGROUP declaration\n",
+	###########################################################################
+	$fileref->printf ("%sstatic const char* _sp_cg_%s_%s_binName(uint64_t point) { \t// SP_COVERGROUP declaration\n",
 			  $prefix,
 			  $covergroupref->name,
 			  $self->name);
-	$fileref->printf ("%s  static string _s_bin_to_name[] = {",$prefix);
+	$fileref->printf ("%s  static const char* _s_bin_to_name[] = {",$prefix);
 	foreach my $bin (@{$self->bins}) {
 	    $fileref->printf ("\"%s\",",$bin->name);
 	}
@@ -530,27 +765,63 @@ sub _write_coverpoint_decl {
 	$fileref->printf ("};\n");
 	$fileref->printf ("%s  return (_s_bin_to_name[point]);\n%s}\n",$prefix,$prefix);
 
+	foreach my $bin (@{$self->bins}) {
+	    my @values = @{$bin->values};
+	    foreach my $value_str (@values) {
+		my $val = $self->validate_value($value_str,$fileref);
+		if ($val < $self->minValue) { $self->minValue($val);}
+		if ($val > $self->maxValue) { $self->maxValue($val);}
+	    }
+
+	    my @ranges = @{$bin->ranges};
+	    foreach my $range (@ranges) {
+		$range =~ /(\S+),(\S+)/;
+		my $hi_str = $1;
+		my $lo_str = $2;
+		my $lo = $self->validate_value($lo_str,$fileref);
+		my $hi = $self->validate_value($hi_str,$fileref);
+
+		if ($lo < $self->minValue) { $self->minValue($lo);}
+		if ($hi < $self->minValue) { $self->minValue($hi);}
+		if ($lo > $self->maxValue) { $self->maxValue($lo);}
+		if ($hi > $self->maxValue) { $self->maxValue($hi);}
+	    }
+	}
+
+	###########################################################################
 	# write the function computing which bin to increment
+	###########################################################################
 	if (($self->minValue < 0) || ($self->maxValue > MAX_BIN_LOOKUP_SIZE)) {
 
 	    $fileref->printf ("%sint _sp_cg_%s_%s_computeBin(uint64_t point) { \t// SP_COVERGROUP declaration\n",
 			      $prefix,
 			      $covergroupref->name,
 			      $self->name);
-	    my $bin_num = 0;
+	    $bin_num = 0;
 	    foreach my $bin (@{$self->bins}) {
 		$fileref->printf ("%s  if (0\n",$prefix);
 
 		my @values = @{$bin->values};
-		foreach my $value (@values) {
-		    $fileref->printf ("%s     || (point == %d)\n",$prefix,$value);
+		foreach my $value_str (@values) {
+
+		    # if it's not an enum, then add ULL to allow 64-bit numbers
+		    $value_str .= "ULL" unless ($value_str =~ /^(\w+)::(\w+)$/);
+
+		    $fileref->printf ("%s     || (point == %s)\n",$prefix,$value_str);
 		}
 		my @ranges = @{$bin->ranges};
 		foreach my $range (@ranges) {
-		    $range =~ /(\d+):(\d+)/;
-		    my $hi = $1;
-		    my $lo = $2;
-		    $fileref->printf ("%s     || ((point >= %d) && (point <= %d))\n", $prefix,$lo, $hi);
+		    $range =~ /(\S+),(\S+)/;
+		    my $hi_str = $1;
+		    my $lo_str = $2;
+		    my $lo = $self->validate_value($lo_str,$fileref);
+		    my $hi = $self->validate_value($hi_str,$fileref);
+
+		    # if it's not an enum, then add ULL to allow 64-bit numbers
+		    $lo .= "ULL" unless ($lo =~ /^(\w+)::(\w+)$/);
+		    $hi .= "ULL" unless ($hi =~ /^(\w+)::(\w+)$/);
+
+		    $fileref->printf ("%s     || ((point >= %s) && (point <= %s))\n", $prefix,$lo, $hi);
 		}
 
 		if ($bin->isIllegal) {
@@ -577,14 +848,17 @@ sub _write_coverpoint_decl {
 	    my $bin_num = 0;
 	    foreach my $bin (@{$self->bins}) {
 		my @values = @{$bin->values};
-		foreach my $value (@values) {
+		foreach my $value_str (@values) {
+		    my $value = $self->validate_value($value_str,$fileref);
 		    $lookupTable[$value] = $bin_num;
 		}
 		my @ranges = @{$bin->ranges};
 		foreach my $range (@ranges) {
-		    $range =~ /(\d+):(\d+)/;
-		    my $hi = $1;
-		    my $lo = $2;
+		    $range =~ /(\S+),(\S+)/;
+		    my $hi_str = $1;
+		    my $lo_str = $2;
+		    my $lo = $self->validate_value($lo_str,$fileref);
+		    my $hi = $self->validate_value($hi_str,$fileref);
 		    for (my $i = $lo; $i <= $hi; $i++) {
 			$lookupTable[$i] = $bin_num;
 		    }
@@ -616,6 +890,9 @@ sub _write_coverpoint_ctor {
 
     # if self->page is undefined, use group page
     my $page = $self->page || $covergroupref->page;
+    $page =~ s/"//g;
+    $page = "{no-page}" if $page eq '';
+    $page = "sp_group/${page}/".$self->name;
 
     # if neither exists, use empty quotes
     my $description = $self->description || "\"\"";
@@ -630,26 +907,48 @@ sub _write_coverpoint_ctor {
 	my $indent = "";
 	foreach my $dimension (@dimensions) {
 	    $indent .= "  "; # indent two more spaces
+
+	    my $num_bins = $dimension->num_bins;
+	    if ($dimension->defaultName eq "") {
+		# don't make a bin for default unless it was specifically called for
+		$num_bins--;
+	    }
+
 	    $fileref->printf("%sfor(int _sp_cg_%s=0;_sp_cg_%s<%d;_sp_cg_%s++) {\n",
 			     $indent,$dimension->name,$dimension->name,
-			     $dimension->num_bins,$dimension->name);
+			     $num_bins,$dimension->name);
 	}
 	$indent .= "  ";
-	$fileref->printf('%sSP_COVER_INSERT(&_sp_cg_%s_%s',
+	# don't insert illegals and ignores
+	my @ignoreVars;
+	foreach my $dimension (@dimensions) {
+	    my $dimname = $dimension->name;
+	    push @ignoreVars, "_sp_cg_${dimname}";
+	}
+	$fileref->printf("%sif (!_sp_cg_%s_%s_ignored(%s) && !_sp_cg_%s_%s_illegal(%s)) {\n",
+			 $indent,
+			 $covergroupref->name,
+			 $self->name,
+			 join(', ',@ignoreVars),
+			 $covergroupref->name,
+			 $self->name,
+			 join(', ',@ignoreVars));
+	$fileref->printf('%s  SP_COVER_INSERT(&_sp_cg_%s_%s',
 			 $indent,
 			 $covergroupref->name,
 			 $self->name);
 	foreach my $dimension (@dimensions) {
 	    $fileref->printf ("[_sp_cg_%s]",$dimension->name);
 	}
-	$fileref->printf(',"filename","%s"', $fileref->name);
+	$fileref->printf(',"filename","%s"', $self->filename);
+	$fileref->printf(',"lineno","%s"', $self->lineno);
 	$fileref->printf(',"groupname","%s"', $covergroupref->name);
 	$fileref->printf(',"per_instance","%s"', $covergroupref->per_instance);
-	$fileref->printf(',"comment",%s', $description); # quotes already present
+	$fileref->printf(',"groupcmt",%s', $description); # quotes already present
 	$fileref->printf(',"pointname","%s"', $self->name);
 	$fileref->printf(',"hier",name()');
 	# fields so the auto-table-generation code will recognize it
-	$fileref->printf(',"page", %s', $page); # quotes already present
+	$fileref->printf(',"page","%s"', $page);
 
 	# FIXME old-style
 	$fileref->printf(',"table", "%s"', $self->name);
@@ -693,30 +992,33 @@ sub _write_coverpoint_ctor {
 	}
 	$fileref->printf(");");
 	$fileref->printf("\n");
+	$fileref->printf("%s}\n",$indent);
 	foreach my $dimension (@dimensions) {
 	    $fileref->printf("}\n");
 	}
-    } else {
-	# FIXME handle ignore and illegal here
-	$fileref->printf("{ for(int i=0;i<%d;i++) {\n",$self->num_bins);
-	$fileref->printf('    ');
+    } elsif (!$self->crossMember) {
+	my $num_bins = $self->num_bins;
+	if ($self->defaultName eq "") {
+	    # don't make a bin for default unless it was specifically called for
+	    $num_bins--;
+	}
+	$fileref->printf("{ for(int i=0;i<%d;i++) {\n",$num_bins);
+	$fileref->printf("    if (!_sp_cg_%s_%s_ignored(i) && !_sp_cg_%s_%s_illegal(i)) {\n",
+			 $covergroupref->name, $self->name,
+			 $covergroupref->name, $self->name);
+	$fileref->printf('      ');
 	$fileref->printf('SP_COVER_INSERT(&_sp_cg_%s_%s[i]',
 			 $covergroupref->name,
 			 $self->name);
-	$fileref->printf(',"filename","%s"', $fileref->name);
+	$fileref->printf(',"filename","%s"', $self->filename);
+	$fileref->printf(',"lineno","%s"', $self->lineno);
 	$fileref->printf(',"groupname","%s"', $covergroupref->name);
 	$fileref->printf(',"per_instance","%s"', $covergroupref->per_instance);
-	$fileref->printf(',"comment",%s', $description); # quotes already present
+	$fileref->printf(',"groupcmt",%s', $description); # quotes already present
 	$fileref->printf(',"pointname","%s"', $self->name);
 	$fileref->printf(',"hier",name()');
-	$fileref->printf(',"ignore",_sp_cg_%s_%s_ignoreStr(i)',
-			 $covergroupref->name,
-			 $self->name);
-	$fileref->printf(',"illegal",_sp_cg_%s_%s_illegalStr(i)',
-			 $covergroupref->name,
-			 $self->name);
 	# fields so the auto-table-generation code will recognize it
-	$fileref->printf(',"page", %s', $page); # quotes already present
+	$fileref->printf(',"page","%s"', $page);
 	$fileref->printf(',"table", "%s"', $self->name);
 	$fileref->printf(',"row0",_sp_cg_%s_%s_binName(i)',
 			 $covergroupref->name,
@@ -725,7 +1027,9 @@ sub _write_coverpoint_ctor {
 			 $self->name);
 	$fileref->printf(");");
 	$fileref->printf("\n");
-	$fileref->printf("} }\n");
+	$fileref->printf("} } }\n");
+    } else {
+	# else this is a 1-d which is a member of a cross; don't insert any points
     }
 }
 
@@ -751,7 +1055,7 @@ SystemPerl is part of the L<http://www.veripool.org/> free SystemC software
 tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/systemperl>.
 
-Copyright 2001-2008 by Wilson Snyder.  This package is free software; you
+Copyright 2001-2009 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License or the Perl Artistic License.
 
@@ -764,4 +1068,3 @@ Bobby Woods-Corwin <me@alum.mit.edu>
 
 L<SystemC::Netlist::Module>
 L<SystemC::Netlist::CoverGroup>
-
