@@ -1,11 +1,11 @@
-// $Id: SpCommon.h 60123 2008-09-04 15:23:30Z wsnyder $ -*- SystemC -*-
+// -*- SystemC -*-
 //=============================================================================
 //
 // THIS MODULE IS PUBLICLY LICENSED
 //
-// Copyright 2001-2008 by Wilson Snyder.  This program is free software;
+// Copyright 2001-2009 by Wilson Snyder.  This program is free software;
 // you can redistribute it and/or modify it under the terms of either the GNU
-// General Public License or the Perl Artistic License.
+// Lesser General Public License or the Perl Artistic License.
 //
 // This is distributed in the hope that it will be useful, but WITHOUT ANY
 // WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -84,6 +84,23 @@
 # define SP_IF_COVER(stmts) do { if(0) { stmts ; } } while(0)
 #endif
 
+//********************************************************************
+// Simple classes.  If get bigger, move to optional include
+
+/// Templated class which constructs to zero.
+/// Originally used for easy pre-zeroing of data used for SpCoverage.
+/// SystemPerl 1.301 and later always zero the points when they are added.
+template <class T> class SpZeroed { public:
+    T m_v;
+    SpZeroed(): m_v(0) {};
+    inline operator const T () const { return m_v; };
+    inline SpZeroed& operator++() {++m_v; return *this;};	// prefix
+    // There is no post-increment; pre-increment may be faster.
+};
+
+/// Uint32_t which constructs to zero.  (Backward compatible)
+typedef SpZeroed<uint32_t> SpUInt32Zeroed;
+
 //=============================================================================
 
 #endif // guard
